/***********************************************************************************************************************************
Test Build Config
***********************************************************************************************************************************/
#include "common/harnessStorage.h"

/***********************************************************************************************************************************
Test Run
***********************************************************************************************************************************/
void
testRun(void)
{
    FUNCTION_HARNESS_VOID();

    // Create default storage object for testing
    Storage *storageTest = storagePosixNewP(TEST_PATH_STR, .write = true);

    // *****************************************************************************************************************************
    if (testBegin("bldCfgParse() and bldCfgRender()"))
    {
        // -------------------------------------------------------------------------------------------------------------------------
        TEST_TITLE("command parse errors");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            "command:\n"
            "  archive-get:\n"
            "    bogus: test\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "unknown command definition 'bogus'");

        #define TEST_COMMAND_VALID                                                                                                 \
            "command:\n"                                                                                                           \
            "  archive-get:\n"                                                                                                     \
            "    internal: true\n"                                                                                                 \
            "\n"

        // -------------------------------------------------------------------------------------------------------------------------
        TEST_TITLE("option group parse errors");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            "optionGroup:\n"
            "  repo:\n"
            "    bogus: test\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "unknown option group definition 'bogus'");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            "optionGroup:\n"
            "  repo: \n"
            "    prefix: test\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "option group 'repo' requires 'indexTotal'");

        #define TEST_OPTION_GROUP_VALID                                                                                            \
            "optionGroup:\n"                                                                                                       \
            "  repo:\n"                                                                                                            \
            "    indexTotal: 2\n"                                                                                                  \
            "\n"

        // -------------------------------------------------------------------------------------------------------------------------
        TEST_TITLE("option parse errors");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    bogus: test\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "unknown option definition 'bogus'");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    section: global\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "option 'config' requires 'type'");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    depend:\n"
            "      bogus: test\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "unknown depend definition 'bogus'");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    deprecate:\n"
            "      old: {bogus: test}\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "unknown deprecate definition 'bogus'");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    command:\n"
            "      backup:\n"
            "        bogus: test\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "unknown option command definition 'bogus'");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    depend: bogus\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "dependency inherited from option 'bogus' before it is defined");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    type: string\n"
            "    command:\n"
            "      bogus: {}\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "invalid command 'bogus' in option 'config' command list");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    type: string\n"
            "    depend:\n"
            "      option: bogus\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "dependency on undefined option 'bogus'");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    type: string\n"
            "    group: bogus\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "option 'config' has invalid group 'bogus'");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  stanza:\n"
            "    type: string\n"
            "  config:\n"
            "    type: string\n"
            "    depend:\n"
            "      option: online\n"
            "  online:\n"
            "    type: boolean\n"
            "    depend:\n"
            "      option: config\n");

        TEST_ERROR(
            bldCfgParse(storageTest), FormatError,
            "unable to resolve dependencies for option(s) 'config, online'\n"
            "HINT: are there circular dependencies?");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    type: string\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "option 'stanza' must exist");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    type: string\n"
            "  stanza:\n"
            "    type: string\n"
            "    depend:\n"
            "      option: config\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "option 'stanza' may not depend on other option");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            TEST_COMMAND_VALID
            TEST_OPTION_GROUP_VALID
            "option:\n"
            "  config:\n"
            "    type: string\n"
            "  stanza:\n"
            "    type: string\n"
            "    command:\n"
            "      archive-get:\n"
            "        depend:\n"
            "          option: config\n");

        TEST_ERROR(bldCfgParse(storageTest), FormatError, "option 'stanza' command 'archive-get' may not depend on other option");

        // -------------------------------------------------------------------------------------------------------------------------
        TEST_TITLE("parse and render config");

        HRN_STORAGE_PUT_Z(
            storageTest, "src/build/config/config.yaml",
            "command:\n"
            "  archive-get:\n"
            "    command-role:\n"
            "      async: {}\n"
            "      local: {}\n"
            "      remote: {}\n"
            "    lock-type: archive\n"
            "    log-file: false\n"
            "    log-level-default: debug\n"
            "    parameter-allowed: true\n"
            "\n"
            "  backup:\n"
            "    internal: true\n"
            "    command-role:\n"
            "      local: {}\n"
            "      remote: {}\n"
            "    lock-type: backup\n"
            "    lock-required: true\n"
            "    lock-remote-required: true\n"
            "\n"
            "  help: {}\n"
            "\n"
            "  version: {}\n"
            "\n"
            "optionGroup:\n"
            "  pg:\n"
            "    indexTotal: 2\n"
            "    prefix: pg\n"
            "  repo:\n"
            "    indexTotal: 2\n"
            "    prefix: repo\n"
            "\n"
            "option:\n"
            "  timeout:\n"
            "    type: time\n"
            "    default: 10\n"
            "    allow-range: [5, 50]\n"
            "\n"
            "  compress-type:\n"
            "    section: global\n"
            "    type: string\n"
            "    default: gz\n"
            "    command:\n"
            "      backup: {}\n"
            "      archive-get:\n"
            "        depend:\n"
            "          option: config\n"
            "    command-role: {}\n"
            "    deprecate:\n"
            "      compress: {}\n"
            "\n"
            "  compress-level:\n"
            "    section: global\n"
            "    type: integer\n"
            "    required: false\n"
            "    allow-range: [0, 9]\n"
            "    command: compress-type\n"
            "    depend: compress-type\n"
            "\n"
            "  compress-level-network:\n"
            "    inherit: compress-level\n"
            "    internal: true\n"
            "    secure: true\n"
            "    default: ~\n"
            "    depend:\n"
            "      option: compress-type\n"
            "      list:\n"
            "        - none\n"
            "        - gz\n"
            "\n"
            "  config:\n"
            "    type: string\n"
            "    default: CFGOPTDEF_CONFIG_PATH \"/\" PROJECT_CONFIG_FILE\n"
            "    default-literal: true\n"
            "    negate: true\n"
            "\n"
            "  log-level-console:\n"
            "    section: global\n"
            "    type: string\n"
            "    default: warn\n"
            "    allow-list:\n"
            "      - off\n"
            "      - error\n"
            "      - warn\n"
            "      - debug1\n"
            "\n"
            "  log-level-file:\n"
            "    section: global\n"
            "    type: string\n"
            "    default: info\n"
            "    allow-list: log-level-console\n"
            "    command:\n"
            "      backup:\n"
            "        internal: true\n"
            "        required: false\n"
            "        default: warn\n"
            "        allow-list:\n"
            "          - off\n"
            "          - warn\n"
            "        depend:\n"
            "          option: log-level-console\n"
            "          list:\n"
            "            - warn\n"
            "        command-role:\n"
            "          main: {}\n"
            "      help: {}\n"
            "      archive-get:\n"
            "        default: ~\n"
            "\n"
            "  stanza:\n"
            "    type: list\n"
            "    command:\n"
            "      archive-get: {}\n"
            "      backup: {}\n"
            "\n"
            "  online:\n"
            "    type: boolean\n"
            "    default: true\n"
            "    command:\n"
            "      backup: {}\n"
            "    command-role:\n"
            "      main: {}\n"
            "    deprecate:\n"
            "      online-old: {}\n"
            "\n"
            "  backup-standby:\n"
            "    section: global\n"
            "    type: boolean\n"
            "    default: false\n"
            "    depend:\n"
            "      option: online\n"
            "    command:\n"
            "      backup: {}\n"
            "    command-role:\n"
            "      async: {}\n"
            "      main: {}\n"
            "    deprecate:\n"
            "      backup-standby-old: {}\n"
            "\n"
            "  pg-path:\n"
            "    section: stanza\n"
            "    group: pg\n"
            "    type: hash\n"
            "    deprecate:\n"
            "      db-path: {index: 1, reset: false}\n"
            "      db?-path: {reset: false}\n");

        TEST_RESULT_VOID(bldCfgRender(storageTest, bldCfgParse(storageTest)), "parse and render");

        // -------------------------------------------------------------------------------------------------------------------------
        TEST_TITLE("check config.auto.h");

        TEST_STORAGE_GET(
            storageTest,
            "src/config/config.auto.h",
            COMMENT_BLOCK_BEGIN "\n"
            "Command and Option Configuration\n"
            "\n"
            "Automatically generated by 'make build-config' -- do not modify directly.\n"
            COMMENT_BLOCK_END "\n"
            "#ifndef CONFIG_CONFIG_AUTO_H\n"
            "#define CONFIG_CONFIG_AUTO_H\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Command constants\n"
            COMMENT_BLOCK_END "\n"
            "#define CFGCMD_ARCHIVE_GET                                          \"archive-get\"\n"
            "#define CFGCMD_BACKUP                                               \"backup\"\n"
            "#define CFGCMD_HELP                                                 \"help\"\n"
            "#define CFGCMD_VERSION                                              \"version\"\n"
            "\n"
            "#define CFG_COMMAND_TOTAL                                           4\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Option group constants\n"
            COMMENT_BLOCK_END "\n"
            "#define CFG_OPTION_GROUP_TOTAL                                      2\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Option constants\n"
            COMMENT_BLOCK_END "\n"
            "#define CFGOPT_BACKUP_STANDBY                                       \"backup-standby\"\n"
            "#define CFGOPT_COMPRESS_LEVEL                                       \"compress-level\"\n"
            "#define CFGOPT_COMPRESS_LEVEL_NETWORK                               \"compress-level-network\"\n"
            "#define CFGOPT_COMPRESS_TYPE                                        \"compress-type\"\n"
            "#define CFGOPT_CONFIG                                               \"config\"\n"
            "#define CFGOPT_LOG_LEVEL_CONSOLE                                    \"log-level-console\"\n"
            "#define CFGOPT_LOG_LEVEL_FILE                                       \"log-level-file\"\n"
            "#define CFGOPT_ONLINE                                               \"online\"\n"
            "#define CFGOPT_STANZA                                               \"stanza\"\n"
            "#define CFGOPT_TIMEOUT                                              \"timeout\"\n"
            "\n"
            "#define CFG_OPTION_TOTAL                                            11\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Option value constants\n"
            COMMENT_BLOCK_END "\n"
            "#define CFGOPTVAL_LOG_LEVEL_CONSOLE_DEBUG1                          STRID6(\"debug1\", 0x7475421441)\n"
            "#define CFGOPTVAL_LOG_LEVEL_CONSOLE_DEBUG1_Z                        \"debug1\"\n"
            "#define CFGOPTVAL_LOG_LEVEL_CONSOLE_ERROR                           STRID5(\"error\", 0x127ca450)\n"
            "#define CFGOPTVAL_LOG_LEVEL_CONSOLE_ERROR_Z                         \"error\"\n"
            "#define CFGOPTVAL_LOG_LEVEL_CONSOLE_OFF                             STRID5(\"off\", 0x18cf0)\n"
            "#define CFGOPTVAL_LOG_LEVEL_CONSOLE_OFF_Z                           \"off\"\n"
            "#define CFGOPTVAL_LOG_LEVEL_CONSOLE_WARN                            STRID5(\"warn\", 0x748370)\n"
            "#define CFGOPTVAL_LOG_LEVEL_CONSOLE_WARN_Z                          \"warn\"\n"
            "\n"
            "#define CFGOPTVAL_LOG_LEVEL_FILE_DEBUG1                             STRID6(\"debug1\", 0x7475421441)\n"
            "#define CFGOPTVAL_LOG_LEVEL_FILE_DEBUG1_Z                           \"debug1\"\n"
            "#define CFGOPTVAL_LOG_LEVEL_FILE_ERROR                              STRID5(\"error\", 0x127ca450)\n"
            "#define CFGOPTVAL_LOG_LEVEL_FILE_ERROR_Z                            \"error\"\n"
            "#define CFGOPTVAL_LOG_LEVEL_FILE_OFF                                STRID5(\"off\", 0x18cf0)\n"
            "#define CFGOPTVAL_LOG_LEVEL_FILE_OFF_Z                              \"off\"\n"
            "#define CFGOPTVAL_LOG_LEVEL_FILE_WARN                               STRID5(\"warn\", 0x748370)\n"
            "#define CFGOPTVAL_LOG_LEVEL_FILE_WARN_Z                             \"warn\"\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Command enum\n"
            COMMENT_BLOCK_END "\n"
            "typedef enum\n"
            "{\n"
            "    cfgCmdArchiveGet,\n"
            "    cfgCmdBackup,\n"
            "    cfgCmdHelp,\n"
            "    cfgCmdVersion,\n"
            "    cfgCmdNone,\n"
            "} ConfigCommand;\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Option group enum\n"
            COMMENT_BLOCK_END "\n"
            "typedef enum\n"
            "{\n"
            "    cfgOptGrpPg,\n"
            "    cfgOptGrpRepo,\n"
            "} ConfigOptionGroup;\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Option enum\n"
            COMMENT_BLOCK_END "\n"
            "typedef enum\n"
            "{\n"
            "    cfgOptBackupStandby,\n"
            "    cfgOptCompressLevel,\n"
            "    cfgOptCompressLevelNetwork,\n"
            "    cfgOptCompressType,\n"
            "    cfgOptConfig,\n"
            "    cfgOptLogLevelConsole,\n"
            "    cfgOptLogLevelFile,\n"
            "    cfgOptOnline,\n"
            "    cfgOptPgPath,\n"
            "    cfgOptStanza,\n"
            "    cfgOptTimeout,\n"
            "} ConfigOption;\n"
            "\n"
            "#endif\n");

        // -------------------------------------------------------------------------------------------------------------------------
        TEST_TITLE("check parse.auto.c");

        TEST_STORAGE_GET(
            storageTest,
            "src/config/parse.auto.c",
            COMMENT_BLOCK_BEGIN "\n"
            "Config Parse Rules\n"
            "\n"
            "Automatically generated by 'make build-config' -- do not modify directly.\n"
            COMMENT_BLOCK_END "\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Command parse data\n"
            COMMENT_BLOCK_END "\n"
            "static const ParseRuleCommand parseRuleCommand[CFG_COMMAND_TOTAL] =\n"
            "{\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_COMMAND\n"
            "    (\n"
            "        PARSE_RULE_COMMAND_NAME(\"archive-get\"),\n"
            "        PARSE_RULE_COMMAND_LOCK_TYPE(lockTypeArchive),\n"
            "        PARSE_RULE_COMMAND_LOG_LEVEL_DEFAULT(logLevelDebug),\n"
            "        PARSE_RULE_COMMAND_PARAMETER_ALLOWED(true),\n"
            "\n"
            "        PARSE_RULE_COMMAND_ROLE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_COMMAND_ROLE(cfgCmdRoleAsync)\n"
            "            PARSE_RULE_COMMAND_ROLE(cfgCmdRoleLocal)\n"
            "            PARSE_RULE_COMMAND_ROLE(cfgCmdRoleMain)\n"
            "            PARSE_RULE_COMMAND_ROLE(cfgCmdRoleRemote)\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_COMMAND\n"
            "    (\n"
            "        PARSE_RULE_COMMAND_NAME(\"backup\"),\n"
            "        PARSE_RULE_COMMAND_LOCK_REQUIRED(true),\n"
            "        PARSE_RULE_COMMAND_LOCK_REMOTE_REQUIRED(true),\n"
            "        PARSE_RULE_COMMAND_LOCK_TYPE(lockTypeBackup),\n"
            "        PARSE_RULE_COMMAND_LOG_FILE(true),\n"
            "        PARSE_RULE_COMMAND_LOG_LEVEL_DEFAULT(logLevelInfo),\n"
            "\n"
            "        PARSE_RULE_COMMAND_ROLE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_COMMAND_ROLE(cfgCmdRoleLocal)\n"
            "            PARSE_RULE_COMMAND_ROLE(cfgCmdRoleMain)\n"
            "            PARSE_RULE_COMMAND_ROLE(cfgCmdRoleRemote)\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_COMMAND\n"
            "    (\n"
            "        PARSE_RULE_COMMAND_NAME(\"help\"),\n"
            "        PARSE_RULE_COMMAND_LOCK_TYPE(lockTypeNone),\n"
            "        PARSE_RULE_COMMAND_LOG_FILE(true),\n"
            "        PARSE_RULE_COMMAND_LOG_LEVEL_DEFAULT(logLevelInfo),\n"
            "\n"
            "        PARSE_RULE_COMMAND_ROLE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_COMMAND_ROLE(cfgCmdRoleMain)\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_COMMAND\n"
            "    (\n"
            "        PARSE_RULE_COMMAND_NAME(\"version\"),\n"
            "        PARSE_RULE_COMMAND_LOCK_TYPE(lockTypeNone),\n"
            "        PARSE_RULE_COMMAND_LOG_FILE(true),\n"
            "        PARSE_RULE_COMMAND_LOG_LEVEL_DEFAULT(logLevelInfo),\n"
            "\n"
            "        PARSE_RULE_COMMAND_ROLE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_COMMAND_ROLE(cfgCmdRoleMain)\n"
            "        ),\n"
            "    ),\n"
            "};\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Option group parse data\n"
            COMMENT_BLOCK_END "\n"
            "static const ParseRuleOptionGroup parseRuleOptionGroup[CFG_OPTION_GROUP_TOTAL] =\n"
            "{\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION_GROUP\n"
            "    (\n"
            "        PARSE_RULE_OPTION_GROUP_NAME(\"pg\"),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION_GROUP\n"
            "    (\n"
            "        PARSE_RULE_OPTION_GROUP_NAME(\"repo\"),\n"
            "    ),\n"
            "};\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Option parse data\n"
            COMMENT_BLOCK_END "\n"
            "static const ParseRuleOption parseRuleOption[CFG_OPTION_TOTAL] =\n"
            "{\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"backup-standby\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeBoolean),\n"
            "        PARSE_RULE_OPTION_REQUIRED(true),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionGlobal),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_OPTIONAL_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_OPTIONAL_DEPEND(cfgOptOnline),\n"
            "            PARSE_RULE_OPTION_OPTIONAL_DEFAULT(\"0\"),\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"compress-level\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeInteger),\n"
            "        PARSE_RULE_OPTION_REQUIRED(false),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionGlobal),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_ASYNC_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_LOCAL_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_REMOTE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_OPTIONAL_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_OPTIONAL_ALLOW_RANGE(0, 9),\n"
            "\n"
            "            PARSE_RULE_OPTION_OPTIONAL_COMMAND_OVERRIDE\n"
            "            (\n"
            "                PARSE_RULE_OPTION_OPTIONAL_COMMAND(cfgCmdArchiveGet),\n"
            "\n"
            "                PARSE_RULE_OPTION_OPTIONAL_DEPEND(cfgOptConfig),\n"
            "            )\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"compress-level-network\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeInteger),\n"
            "        PARSE_RULE_OPTION_REQUIRED(false),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionGlobal),\n"
            "        PARSE_RULE_OPTION_SECURE(true),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_ASYNC_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_LOCAL_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_REMOTE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_OPTIONAL_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_OPTIONAL_ALLOW_RANGE(0, 9),\n"
            "            PARSE_RULE_OPTION_OPTIONAL_DEPEND_LIST\n"
            "            (\n"
            "                cfgOptCompressType,\n"
            "                \"none\",\n"
            "                \"gz\"\n"
            "            ),\n"
            "\n"
            "            PARSE_RULE_OPTION_OPTIONAL_COMMAND_OVERRIDE\n"
            "            (\n"
            "                PARSE_RULE_OPTION_OPTIONAL_COMMAND(cfgCmdArchiveGet),\n"
            "\n"
            "                PARSE_RULE_OPTION_OPTIONAL_DEPEND(cfgOptConfig),\n"
            "            )\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"compress-type\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeString),\n"
            "        PARSE_RULE_OPTION_REQUIRED(true),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionGlobal),\n"
            "\n"
            "        PARSE_RULE_OPTION_OPTIONAL_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_OPTIONAL_DEFAULT(\"gz\"),\n"
            "\n"
            "            PARSE_RULE_OPTION_OPTIONAL_COMMAND_OVERRIDE\n"
            "            (\n"
            "                PARSE_RULE_OPTION_OPTIONAL_COMMAND(cfgCmdArchiveGet),\n"
            "\n"
            "                PARSE_RULE_OPTION_OPTIONAL_DEPEND(cfgOptConfig),\n"
            "            )\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"config\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeString),\n"
            "        PARSE_RULE_OPTION_REQUIRED(true),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionCommandLine),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_ASYNC_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_LOCAL_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_REMOTE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_OPTIONAL_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_OPTIONAL_DEFAULT(CFGOPTDEF_CONFIG_PATH \"/\" PROJECT_CONFIG_FILE),\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"log-level-console\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeString),\n"
            "        PARSE_RULE_OPTION_REQUIRED(true),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionGlobal),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_ASYNC_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_LOCAL_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_REMOTE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_OPTIONAL_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_OPTIONAL_ALLOW_LIST\n"
            "            (\n"
            "                \"off\",\n"
            "                \"error\",\n"
            "                \"warn\",\n"
            "                \"debug1\"\n"
            "            ),\n"
            "\n"
            "            PARSE_RULE_OPTION_OPTIONAL_DEFAULT(\"warn\"),\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"log-level-file\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeString),\n"
            "        PARSE_RULE_OPTION_REQUIRED(true),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionGlobal),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdHelp)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_ASYNC_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_LOCAL_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_REMOTE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_OPTIONAL_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_OPTIONAL_ALLOW_LIST\n"
            "            (\n"
            "                \"off\",\n"
            "                \"error\",\n"
            "                \"warn\",\n"
            "                \"debug1\"\n"
            "            ),\n"
            "\n"
            "            PARSE_RULE_OPTION_OPTIONAL_DEFAULT(\"info\"),\n"
            "\n"
            "            PARSE_RULE_OPTION_OPTIONAL_COMMAND_OVERRIDE\n"
            "            (\n"
            "                PARSE_RULE_OPTION_OPTIONAL_COMMAND(cfgCmdBackup),\n"
            "\n"
            "                PARSE_RULE_OPTION_OPTIONAL_ALLOW_LIST\n"
            "                (\n"
            "                    \"off\",\n"
            "                    \"warn\"\n"
            "                ),\n"
            "\n"
            "                PARSE_RULE_OPTION_OPTIONAL_DEPEND_LIST\n"
            "                (\n"
            "                    cfgOptLogLevelConsole,\n"
            "                    \"warn\"\n"
            "                ),\n"
            "\n"
            "                PARSE_RULE_OPTION_OPTIONAL_DEFAULT(\"warn\"),\n"
            "                PARSE_RULE_OPTION_OPTIONAL_REQUIRED(false),\n"
            "            )\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"online\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeBoolean),\n"
            "        PARSE_RULE_OPTION_REQUIRED(true),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionCommandLine),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_OPTIONAL_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_OPTIONAL_DEFAULT(\"1\"),\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"pg-path\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeHash),\n"
            "        PARSE_RULE_OPTION_REQUIRED(true),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionStanza),\n"
            "        PARSE_RULE_OPTION_MULTI(true),\n"
            "        PARSE_RULE_OPTION_GROUP_MEMBER(true),\n"
            "        PARSE_RULE_OPTION_GROUP_ID(cfgOptGrpPg),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_ASYNC_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_LOCAL_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_REMOTE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"stanza\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeList),\n"
            "        PARSE_RULE_OPTION_REQUIRED(true),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionCommandLine),\n"
            "        PARSE_RULE_OPTION_MULTI(true),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_ASYNC_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_LOCAL_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_REMOTE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "    ),\n"
            "\n"
            COMMENT_SEPARATOR "\n"
            "    PARSE_RULE_OPTION\n"
            "    (\n"
            "        PARSE_RULE_OPTION_NAME(\"timeout\"),\n"
            "        PARSE_RULE_OPTION_TYPE(cfgOptTypeTime),\n"
            "        PARSE_RULE_OPTION_REQUIRED(true),\n"
            "        PARSE_RULE_OPTION_SECTION(cfgSectionCommandLine),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_MAIN_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_ASYNC_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_LOCAL_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_COMMAND_ROLE_REMOTE_VALID_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdArchiveGet)\n"
            "            PARSE_RULE_OPTION_COMMAND(cfgCmdBackup)\n"
            "        ),\n"
            "\n"
            "        PARSE_RULE_OPTION_OPTIONAL_LIST\n"
            "        (\n"
            "            PARSE_RULE_OPTION_OPTIONAL_ALLOW_RANGE(5000, 50000),\n"
            "            PARSE_RULE_OPTION_OPTIONAL_DEFAULT(\"10000\"),\n"
            "        ),\n"
            "    ),\n"
            "};\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Option data for getopt_long()\n"
            COMMENT_BLOCK_END "\n"
            "static const struct option optionList[] =\n"
            "{\n"
            "    // backup-standby option and deprecations\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"backup-standby\",\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptBackupStandby,\n"
            "    },\n"
            "    {\n"
            "        .name = \"no-backup-standby\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_NEGATE_FLAG | cfgOptBackupStandby,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-backup-standby\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_RESET_FLAG | cfgOptBackupStandby,\n"
            "    },\n"
            "    {\n"
            "        .name = \"backup-standby-old\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_DEPRECATE_FLAG | cfgOptBackupStandby,\n"
            "    },\n"
            "    {\n"
            "        .name = \"no-backup-standby-old\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_DEPRECATE_FLAG | PARSE_NEGATE_FLAG | cfgOptBackupStandby,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-backup-standby-old\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_DEPRECATE_FLAG | PARSE_RESET_FLAG | cfgOptBackupStandby,\n"
            "    },\n"
            "\n"
            "    // compress-level option\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"compress-level\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptCompressLevel,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-compress-level\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_RESET_FLAG | cfgOptCompressLevel,\n"
            "    },\n"
            "\n"
            "    // compress-level-network option\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"compress-level-network\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptCompressLevelNetwork,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-compress-level-network\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_RESET_FLAG | cfgOptCompressLevelNetwork,\n"
            "    },\n"
            "\n"
            "    // compress-type option and deprecations\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"compress-type\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptCompressType,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-compress-type\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_RESET_FLAG | cfgOptCompressType,\n"
            "    },\n"
            "    {\n"
            "        .name = \"compress\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_DEPRECATE_FLAG | cfgOptCompressType,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-compress\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_DEPRECATE_FLAG | PARSE_RESET_FLAG | cfgOptCompressType,\n"
            "    },\n"
            "\n"
            "    // config option\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"config\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptConfig,\n"
            "    },\n"
            "    {\n"
            "        .name = \"no-config\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_NEGATE_FLAG | cfgOptConfig,\n"
            "    },\n"
            "\n"
            "    // log-level-console option\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"log-level-console\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptLogLevelConsole,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-log-level-console\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_RESET_FLAG | cfgOptLogLevelConsole,\n"
            "    },\n"
            "\n"
            "    // log-level-file option\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"log-level-file\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptLogLevelFile,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-log-level-file\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_RESET_FLAG | cfgOptLogLevelFile,\n"
            "    },\n"
            "\n"
            "    // online option and deprecations\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"online\",\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptOnline,\n"
            "    },\n"
            "    {\n"
            "        .name = \"online-old\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_DEPRECATE_FLAG | cfgOptOnline,\n"
            "    },\n"
            "\n"
            "    // pg-path option and deprecations\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"pg1-path\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | (0 << PARSE_KEY_IDX_SHIFT) | cfgOptPgPath,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-pg1-path\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_RESET_FLAG | (0 << PARSE_KEY_IDX_SHIFT) | cfgOptPgPath,\n"
            "    },\n"
            "    {\n"
            "        .name = \"db-path\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_DEPRECATE_FLAG | (0 << PARSE_KEY_IDX_SHIFT) | cfgOptPgPath,\n"
            "    },\n"
            "    {\n"
            "        .name = \"db1-path\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_DEPRECATE_FLAG | (0 << PARSE_KEY_IDX_SHIFT) | cfgOptPgPath,\n"
            "    },\n"
            "    {\n"
            "        .name = \"pg2-path\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | (1 << PARSE_KEY_IDX_SHIFT) | cfgOptPgPath,\n"
            "    },\n"
            "    {\n"
            "        .name = \"reset-pg2-path\",\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_RESET_FLAG | (1 << PARSE_KEY_IDX_SHIFT) | cfgOptPgPath,\n"
            "    },\n"
            "    {\n"
            "        .name = \"db2-path\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | PARSE_DEPRECATE_FLAG | (1 << PARSE_KEY_IDX_SHIFT) | cfgOptPgPath,\n"
            "    },\n"
            "\n"
            "    // stanza option\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"stanza\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptStanza,\n"
            "    },\n"
            "\n"
            "    // timeout option\n"
            COMMENT_SEPARATOR "\n"
            "    {\n"
            "        .name = \"timeout\",\n"
            "        .has_arg = required_argument,\n"
            "        .val = PARSE_OPTION_FLAG | cfgOptTimeout,\n"
            "    },\n"
            "    // Terminate option list\n"
            "    {\n"
            "        .name = NULL\n"
            "    }\n"
            "};\n"
            "\n"
            COMMENT_BLOCK_BEGIN "\n"
            "Order for option parse resolution\n"
            COMMENT_BLOCK_END "\n"
            "static const ConfigOption optionResolveOrder[] =\n"
            "{\n"
            "    cfgOptStanza,\n"
            "    cfgOptConfig,\n"
            "    cfgOptLogLevelConsole,\n"
            "    cfgOptLogLevelFile,\n"
            "    cfgOptOnline,\n"
            "    cfgOptPgPath,\n"
            "    cfgOptTimeout,\n"
            "    cfgOptBackupStandby,\n"
            "    cfgOptCompressLevel,\n"
            "    cfgOptCompressType,\n"
            "    cfgOptCompressLevelNetwork,\n"
            "};\n");
    }

    FUNCTION_HARNESS_RETURN_VOID();
}
